/**
 * ProjectLaogai
 *
 * Copyright 2019  <seil0@mosad.xyz>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 */

package org.mosad.seil0.projectlaogai.fragments

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import com.afollestad.materialdialogs.MaterialDialog
import kotlinx.android.synthetic.main.fragment_timetable.*
import org.jetbrains.anko.doAsync
import org.jetbrains.anko.uiThread
import org.mosad.seil0.projectlaogai.R
import org.mosad.seil0.projectlaogai.controller.CacheController.Companion.timetables
import org.mosad.seil0.projectlaogai.hsoparser.DataTypes
import org.mosad.seil0.projectlaogai.hsoparser.NotRetardedCalendar
import org.mosad.seil0.projectlaogai.hsoparser.TimetableWeek
import org.mosad.seil0.projectlaogai.uicomponents.*
import java.text.SimpleDateFormat
import java.util.*

/**
 * The timetable controller class
 * contains all needed parts to display and the timetable detail screen
 */
class TimeTableFragment : Fragment() {

    private val formatter = SimpleDateFormat("E dd.MM", Locale.getDefault()) // TODO change to android call when min api is 24

    override fun onCreateView(inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?): View? {

        val view: View = inflater.inflate(R.layout.fragment_timetable, container, false)

        if (timetables[0].days.isNotEmpty() && timetables[1].days.isNotEmpty()) {
            addInitWeeks()
        } else {
            MaterialDialog(context!!)
                .title(R.string.error)
                .message(R.string.timetable_error)
                .show()
        }

        return view
    }

    /**
     * add the current and next weeks lessons
     */
    private fun addInitWeeks() = doAsync {
        val dayIndex = NotRetardedCalendar().getDayOfWeekIndex()
        val calendar = Calendar.getInstance()

        println(formatter.format(calendar.time))

        // add current week
        addWeek(dayIndex, 5, timetables[0], calendar).get()

        // add next week
        addWeek(0, dayIndex - 1, timetables[1], calendar)
    }

    private fun addWeek(dayStart: Int, dayEnd: Int, timetable: TimetableWeek, calendar: Calendar) = doAsync {
        uiThread {

            for (day in dayStart..dayEnd) {
                var helpLesson = LessonLinearLayout(context)
                val dayCardView = DayCardView(context!!)
                dayCardView.setDayHeading(formatter.format(calendar.time))
                println("1: $day , ${formatter.format(calendar.time)}")

                // for each timeslot of the day
                for ((tsIndex, timeslot) in timetable.days[day].timeslots.withIndex()) {

                    for(lesson in timeslot) {

                        if(lesson.lessonSubject.isNotEmpty()) {

                            val lessonLayout = LessonLinearLayout(context)
                            lessonLayout.setLesson(lesson, DataTypes().getTime()[tsIndex])
                            dayCardView.getLinLayoutDay().addView(lessonLayout)

                            if (lesson != timeslot.last())
                                lessonLayout.disableDivider()

                            helpLesson = lessonLayout
                        }
                    }
                }

                println("2: $day , ${formatter.format(calendar.time)}")

                helpLesson.disableDivider()
                calendar.add(Calendar.DATE, 1)

                // if there are no lessons don't show the dayCardView
                if (dayCardView.getLinLayoutDay().childCount > 1)
                    linLayout_Timetable.addView(dayCardView)

            }

            calendar.add(Calendar.DATE, 1) // before this we are at a sunday (no lecture on sundays!)
        }
    }

}
